import java.awt.*;
import aLibrary.*;

/**  DrawingTool is like a drawing pen that
    can be moved around a drawing canvas
    This is similar to a turtle from the
    LOGO programming language.

    Class Invariant
        parentContainer == window
        AND  up <= direction <= left */


public class DrawingTool extends AView {
    private static AWindow window;
    private int locX, locY;
    private boolean isDrawing;
    private int direction;
    private final int up = 0;
    private final int right = 1;
    private final int down = 2;
    private final int left = 3;


    /**  postcondition
            old window != null  IMPLIES  window == old window
        AND  window.getWidth()==400  AND  window.getHeight()==300
        AND  getX()==195  AND  getY()==141  AND getWidth()==getHeight()==9
        AND  isDrawing
        AND  direction == up
        AND  getColor() == Color.green  */
    public DrawingTool()  {
        super(96, 66, 9, 9);
        if  ( AWindow.getWindowCounter() == 0 )   {
            window = new AWindow(10, 10, 200, 150);
            window.setTitle( "Drawing Canvas" );
        }
        locX = 100;
        locY = 75;
        draw();
        direction = up;
        place(window);
        repaint();
        window.setVisible(true);
        window.repaint();
    }

    /**  postconditon
            isDrawing
            AND  getColor == Color.green */
    public void draw()   {
        isDrawing = true;
        setColor( Color.green );
        window.repaint();
        try {
            Thread.sleep(10);
        } catch (InterruptedException e) {}
    }

    /**  postconditon
            NOT isDrawing
            AND  getColor == Color.red */
    public void dontDraw()   {
        isDrawing = false;
        setColor( Color.red );
        window.repaint();
        try {
            Thread.sleep(10);
        } catch (InterruptedException e) {}
    }

    /**  postconditon
            direction == a 90 clockwise rotation from old direction */
    public void turnClockwise()   {
        direction = (direction + 1) % 4;
        repaint();
        window.repaint();
        try {
            Thread.sleep(20);
        } catch (InterruptedException e) {}
    }

    /**  postconditon
            direction == a 90 counterclockwise rotation from old direction */
    public void turnCounterclockwise()   {
        if (direction == 0)   {
            direction = 4;
        }
        direction--;
        adjustArrow();
        window.repaint();
        try {
            Thread.sleep(10);
        } catch (InterruptedException e) {}
    }

    /**  postconditon
            the arrow is placed so that it is positioned in front
            of the (locX, locY) position. */
    private void adjustArrow()   {
        if (direction == up)   {
            setLocation(locX-4, locY-9);
        } else if (direction == right)   {
            setLocation(locX, locY-4);
        } else if (direction == down)   {
            setLocation(locX-4, locY);
        } else   {
            setLocation(locX-9, locY-4);
        }
        repaint();
    }

    /**  postconditon
            (locX, locY) is a point that is d units from (old locX, old locY)
            moving according to direction
            AND  isDrawing IMPLIES a line segment is drawn connecting (locX, locY)
                    with (old locX, old locY) */
    public void moveForwardBy( int d )   {
        ALine line;
        int newX, newY;
        newX = locX;
        newY = locY;
        if  ( direction == up )   {
            newY = locY - d;
        } else if  ( direction == right )   {
            newX = locX + d;
        } else if  ( direction == down )   {
            newY = locY + d;
        } else    {
            newX = locX - d;
        }
        if  ( isDrawing )   {
            line = new ALine(locX, locY, newX, newY);
            line.place(window);
            line.repaint();
        }
        locX = newX;
        locY = newY;
        adjustArrow();
        repaint();
        window.repaint();
        try {
            Thread.sleep(10);
        } catch (InterruptedException e) {}
    }

    /**  postconditon
            (locX, locY) is a point that is 20 units from (old locX, old locY)
            moving according to direction
            AND  isDrawing IMPLIES a line segment is drawn connecting (locX, locY)
                    with (old locX, old locY) */
    public void moveForward()   {
        moveForwardBy(20);
    }

   /**   postconditon
            this AView is drawn as an arrow using the given getColor() */
   public void stroke(Graphics g)  {
      g.setColor(getColor());
      if ( direction == up )   {
         g.drawLine(1, 6, 4, 0);
         g.drawLine(4, 0, 7, 6);
         g.drawLine(4, 0, 4, 8);
         g.drawLine(3, 2, 5, 2);
         g.drawLine(3, 3, 5, 3);
         g.drawLine(2, 4, 6, 4);
         g.drawLine(2, 5, 6, 5);
      }  else if  ( direction == right ) {
         g.drawLine(2, 1, 8, 4);
         g.drawLine(8, 4, 2, 7);
         g.drawLine(0, 4, 8, 4);
         g.drawLine(6, 3, 6, 5);
         g.drawLine(5, 3, 5, 5);
         g.drawLine(4, 2, 4, 6);
         g.drawLine(3, 2, 3, 6);
      }  else if  ( direction == down ) {
         g.drawLine(1, 2, 4, 8);
         g.drawLine(4, 8, 7, 2);
         g.drawLine(4, 0, 4, 8);
         g.drawLine(3, 6, 5, 6);
         g.drawLine(3, 5, 5, 5);
         g.drawLine(2, 4, 6, 4);
         g.drawLine(2, 3, 6, 3);
      }  else {
         g.drawLine(6, 1, 0, 4);
         g.drawLine(0, 4, 6, 7);
         g.drawLine(0, 4, 8, 4);
         g.drawLine(2, 3, 2, 5);
         g.drawLine(3, 3, 3, 5);
         g.drawLine(4, 2, 4, 6);
         g.drawLine(5, 2, 5, 6);
      }
   }

   /**   postconditon
            this AView is drawn as an arrow using the given getColor() */
   public void fill(Graphics g)  {
      stroke(g);
   }

   /**   postconditon
            window == null */
   public void finalize()  {
      window = null;
   }
}